/*
    Copyright (C) 2008-2010 Stefan Haller

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <config.h>
#include <libxml/xmlmemory.h>
#include <libxml/parser.h>
#include "xml.h"
#include "general.h"

gchar * xmlCharToGChar(xmlChar * str, gboolean dofree)
{
	if (str == NULL)
	{
		return NULL;
	}
	else
	{
		gchar * result = g_strdup((gchar *)str);
		if (dofree)
			xmlFree(str);
		
		return result;
	}
}

struct settings * load_settings()
{
	gboolean first_profile, first_file, first_module;
	struct settings * set = g_new(struct settings, 1);
	struct profile_node * profile;
	struct file_node * file;
	struct module_node * module;
	
	set->interval = 5;
	set->changebyinterval = FALSE;
	set->changeeverylaunch = FALSE;
	#ifdef CACHE_FILE_LIST
	set->cachefilelist = FALSE;
	#endif
	set->mouse_wheel = TRUE;
	set->default_profile = NULL;
	set->profiles = NULL;
	set->modules = NULL;
	
	gchar * filename = get_setting_file();
	
	xmlDocPtr doc;
	xmlNodePtr root, depth1, depth2, depth3;
	xmlChar * str;
	
	doc = xmlParseFile(filename);
	
	if (doc != NULL)
	{
		root = xmlDocGetRootElement(doc);
		if (root != NULL)
		{
			/* depth1*/
			depth1 = root->children;
			
			while (depth1 != NULL)
			{
				if ((!xmlStrcmp(depth1->name, (const xmlChar *)"profiles")))
				{
					/* depth2 */
					depth2 = depth1->children;
					
					first_profile = TRUE;
					
					while (depth2 != NULL)
					{
						if ((!xmlStrcmp(depth2->name, (const xmlChar *)"profile")))
						{
							if (first_profile)
							{
								profile = g_new(struct profile_node, 1);
								set->profiles = profile;
								first_profile = FALSE;
							}
							else
							{
								profile->next = g_new(struct profile_node, 1);
								profile = profile->next;
							}
							
							profile->name = xmlCharToGChar(xmlGetProp(depth2, (const xmlChar *)"name"), TRUE);
							
							profile->next = NULL;
							profile->files = NULL;
							
							/* depth 3*/
							depth3 = depth2->children;
							
							first_file = TRUE;
							
							while (depth3 != NULL)
							{
								if ((!xmlStrcmp(depth3->name, (const xmlChar *)"file")))
								{
									if (first_file)
									{
										file = g_new(struct file_node, 1);
										profile->files = file;
										first_file = FALSE;
									}
									else
									{
										file->next = g_new(struct file_node, 1);
										file = file->next;
									}
									
									file->filename = xmlCharToGChar(xmlGetProp(depth3, (const xmlChar *)"name"), TRUE);
									
									str = xmlGetProp(depth3, (const xmlChar *)"isdir");
									if (str == NULL)
									{
										file->is_dir = FALSE;
									}
									else
									{
										file->is_dir = xmlStrcmp(str, (const xmlChar *)"yes") == 0;
										xmlFree(str);
									}
									
									str = xmlGetProp(depth3, (const xmlChar *)"usesubfolders");
									if (str == NULL)
									{
										file->use_subfolders = FALSE;
									}
									else
									{
										file->use_subfolders = xmlStrcmp(str, (const xmlChar *)"yes") == 0;
										xmlFree(str);
									}
									
									file->next = NULL;
								}
								depth3 = depth3->next;
							}
						}
						
						depth2 = depth2->next;
					}
				}
				
				if ((!xmlStrcmp(depth1->name, (const xmlChar *)"settings")))
				{
					/* depth2 */
					depth2 = depth1->children;
					
					while (depth2 != NULL)
					{
						if ((!xmlStrcmp(depth2->name, (const xmlChar *)"setting")))
						{
							xmlChar * name = xmlGetProp(depth2, (const xmlChar *)"name");
							xmlChar * value = xmlGetProp(depth2, (const xmlChar *)"value");
							
							if ((name != NULL) && (value != NULL))
							{
								if ((!xmlStrcmp(name, (const xmlChar *)"interval")))
								{
									if (atoi((char *)value))
									{
										set->interval = atoi((char *)value);
									}
								}
								if ((!xmlStrcmp(name, (const xmlChar *)"changebyinterval")))
								{
									set->changebyinterval = xmlStrcmp(value, (const xmlChar *)"yes") == 0;
								}
								if ((!xmlStrcmp(name, (const xmlChar *)"changeeverylaunch")))
								{
									set->changeeverylaunch = xmlStrcmp(value, (const xmlChar *)"yes") == 0;
								}
								#ifdef CACHE_FILE_LIST
								if ((!xmlStrcmp(name, (const xmlChar *)"cachefilelist")))
								{
									set->cachefilelist = xmlStrcmp(value, (const xmlChar *)"yes") == 0;
								}
								#endif
								if ((!xmlStrcmp(name, (const xmlChar *)"mousewheel")))
								{
									set->mouse_wheel = xmlStrcmp(value, (const xmlChar *)"yes") == 0;
								}
								if ((!xmlStrcmp(name, (const xmlChar *)"defaultprofile")))
								{
									set->default_profile = xmlCharToGChar(value, FALSE);
								}
							}
							
							xmlFree(name);
							xmlFree(value);
						}
						
						depth2 = depth2->next;
					}
				}
				
				if ((!xmlStrcmp(depth1->name, (const xmlChar *)"modules")))
				{
					/* depth2 */
					depth2 = depth1->children;
					
					first_module = TRUE;
					
					while (depth2 != NULL)
					{
						if ((!xmlStrcmp(depth2->name, (const xmlChar *)"module")))
						{
							if (first_module)
							{
								module = g_new(struct module_node, 1);
								set->modules = module;
								first_module = FALSE;
							}
							else
							{
								module->next = g_new(struct module_node, 1);
								module = module->next;
							}
							
							module->name = xmlCharToGChar(xmlGetProp(depth2, (const xmlChar *)"name"), TRUE);
							
							module->next = NULL;
						}
						
						depth2 = depth2->next;
					}
				}
				depth1 = depth1->next;
			}
		} /* root != NULL */
	} /* doc != NULL */
	
	if (doc != NULL)
	{
		xmlFreeDoc(doc);
	}
	
	xmlCleanupParser();
	#ifdef DEBUG
	xmlMemoryDump();
	#endif
	
	g_free(filename);
	
	return set;
}



void save_settings(struct settings * set)
{
	xmlDocPtr doc = xmlNewDoc((const xmlChar *) "1.0");
	xmlNodePtr root, depth1, depth2, depth3;
	
	struct file_node * file;
	struct profile_node * profile;
	struct module_node * module;
	
	if (doc != NULL)
	{
		root = xmlNewNode(NULL, (const xmlChar *)"desktopnova");
		if (root != NULL)
		{
			xmlDocSetRootElement(doc, root);
			
			depth1 = xmlNewChild(root, NULL, (const xmlChar *)"profiles", NULL);
			
			profile = set->profiles;
			while (profile != NULL)
			{
				depth2 = xmlNewChild(depth1, NULL, (const xmlChar *)"profile", NULL);
				xmlSetProp(depth2, (const xmlChar *)"name", (xmlChar *)profile->name);
				
				file = profile->files;
				while (file != NULL)
				{
					depth3 = xmlNewChild(depth2, NULL, (const xmlChar *)"file", NULL);
					
					xmlSetProp(depth3, (const xmlChar *)"name", (xmlChar *)file->filename);
					
					if (file->is_dir)
					{
						xmlSetProp(depth3, (const xmlChar *)"isdir", (const xmlChar *)"yes");
					}
					else
					{
						xmlSetProp(depth3, (const xmlChar *)"isdir", (const xmlChar *)"no");
					}
					
					if (file->use_subfolders)
					{
						xmlSetProp(depth3, (const xmlChar *)"usesubfolders", (const xmlChar *)"yes");
					}
					else
					{
						xmlSetProp(depth3, (const xmlChar *)"usesubfolders", (const xmlChar *)"no");
					}
					
					file = file->next;
				}
				
				profile = profile->next;
			}			
			
			depth1 = xmlNewChild(root, NULL, (const xmlChar *)"settings", NULL);
			
			depth2 = xmlNewChild(depth1, NULL, (const xmlChar *)"setting", NULL);
			xmlSetProp(depth2, (const xmlChar *)"name", (const xmlChar *)"interval");
			gchar * str = g_strdup_printf("%i", set->interval);
			xmlSetProp(depth2, (const xmlChar *)"value", (xmlChar *)str);
			free(str);
			
			depth2 = xmlNewChild(depth1, NULL, (const xmlChar *)"setting", NULL);
			xmlSetProp(depth2, (const xmlChar *)"name", (const xmlChar *)"changebyinterval");
			if (set->changebyinterval)
			{
				xmlSetProp(depth2, (const xmlChar *)"value", (const xmlChar *)"yes");
			}
			else
			{
				xmlSetProp(depth2, (const xmlChar *)"value", (const xmlChar *)"no");
			}
			
			depth2 = xmlNewChild(depth1, NULL, (const xmlChar *)"setting", NULL);
			xmlSetProp(depth2, (const xmlChar *)"name", (const xmlChar *)"changeeverylaunch");
			if (set->changeeverylaunch)
			{
				xmlSetProp(depth2, (const xmlChar *)"value", (const xmlChar *)"yes");
			}
			else
			{
				xmlSetProp(depth2, (const xmlChar *)"value", (const xmlChar *)"no");
			}
			
			#ifdef CACHE_FILE_LIST
			depth2 = xmlNewChild(depth1, NULL, (const xmlChar *)"setting", NULL);
			xmlSetProp(depth2, (const xmlChar *)"name", (const xmlChar *)"cachefilelist");
			if (set->cachefilelist)
			{
				xmlSetProp(depth2, (const xmlChar *)"value", (const xmlChar *)"yes");
			}
			else
			{
				xmlSetProp(depth2, (const xmlChar *)"value", (const xmlChar *)"no");
			}
			#endif
			
			depth2 = xmlNewChild(depth1, NULL, (const xmlChar *)"setting", NULL);
			xmlSetProp(depth2, (const xmlChar *)"name", (const xmlChar *)"mousewheel");
			if (set->mouse_wheel)
			{
				xmlSetProp(depth2, (const xmlChar *)"value", (const xmlChar *)"yes");
			}
			else
			{
				xmlSetProp(depth2, (const xmlChar *)"value", (const xmlChar *)"no");
			}
			
			depth2 = xmlNewChild(depth1, NULL, (const xmlChar *)"setting", NULL);
			xmlSetProp(depth2, (const xmlChar *)"name", (const xmlChar *)"defaultprofile");
			xmlSetProp(depth2, (const xmlChar *)"value", (xmlChar *)set->default_profile);
			
			depth1 = xmlNewChild(root, NULL, (const xmlChar *)"modules", NULL);
			module = set->modules;
			while (module != NULL)
			{
				depth2 = xmlNewChild(depth1, NULL, (const xmlChar *)"module", NULL);
				xmlSetProp(depth2, (const xmlChar *)"name", (xmlChar *)module->name);
				module = module->next;
			}
			
			gchar * filename = get_setting_file();
			
			FILE * f = fopen(filename, "w");
			xmlDocFormatDump(f, doc, TRUE);
			fclose(f);
		
			g_free(filename);
		}
	}
	
	if (doc != NULL)
	{
		xmlFreeDoc(doc);
	}
	
	xmlCleanupParser();
	#ifdef DEBUG
	xmlMemoryDump();
	#endif
}

void free_settings(struct settings * set)
{
	struct file_node * file;
	struct file_node * file_old;
	struct profile_node * profile;
	struct profile_node * profile_old;
	struct module_node * module;
	struct module_node * module_old;
	
	profile = set->profiles;
	
	while (profile != NULL)
	{
		file = profile->files;
		while (file != NULL)
		{
			g_free(file->filename);
			file_old = file;
			file = file->next;
			g_free(file_old);
		}
		g_free(profile->name);
		profile_old = profile;
		profile = profile->next;
		g_free(profile_old);
	}
	
	module = set->modules;
	while (module != NULL)
	{
		g_free(module->name);
		module_old = module;
		module = module->next;
		g_free(module_old);
	}
	
	
	g_free(set->default_profile);
	
	g_free(set);
}

